@@ -1,4 +1,4 @@
-import { OTHER_REFERENCE_TYPES, REFERENCE_TYPE_VALUES } from './importable';
+import { OTHER_REFERENCE_TYPES, REFERENCE_TYPE_VALUES } from './importable.js';
 
 export const SUGGESTER_ENDPOINTS_SUPPORTING_ALL = [
   ...REFERENCE_TYPE_VALUES,
@@ -12,8 +12,10 @@ export const SUGGESTER_ENDPOINTS = [
   'department',
   'facility',
   'facilityLocationGroup',
-  'invoiceProducts',
+  'bookableLocationGroup',
+  'invoiceProduct',
   'location',
+  'multiReferenceData',
   'patient',
   'patientLabTestCategories',
   'patientLabTestPanelTypes',
@@ -23,4 +25,7 @@ export const SUGGESTER_ENDPOINTS = [
   'programRegistryClinicalStatus',
   'programRegistryCondition',
   'survey',
+  'sensitiveLabTestCategory',
+  'nonSensitiveLabTestCategory',
+  'reportDefinition',
 ];
