@@ -7,6 +7,22 @@ import { selectFacilityIds } from '@tamanu/utils/selectFacilityIds';
 import { GLOBAL_EXCLUDE_TABLES, NON_LOGGED_TABLES, NON_SYNCING_TABLES } from './constants';
 import { SYNC_TICK_FLAGS } from '../../sync/constants';
 
+const tableNameMatch = (schema: string, table: string, matches: string[]) => {
+  const matchTableSchemas = matches
+    .map(match => match.split('.'))
+    .map(([excludeSchema, excludeTable]) => ({ schema: excludeSchema, table: excludeTable }));
+  const wholeSchemaMatches = matchTableSchemas
+    .filter(({ table: matchTable }) => matchTable === '*')
+    .map(({ schema: matchSchema }) => matchSchema);
+  if (wholeSchemaMatches.includes(schema)) {
+    return true;
+  }
+
+  return matchTableSchemas.some(
+    ({ schema: matchSchema, table: matchTable }) => schema === matchSchema && table === matchTable,
+  );
+};
+
 export const tablesWithoutColumn = (
   sequelize: Sequelize,
   column: string,
@@ -37,7 +53,7 @@ export const tablesWithoutColumn = (
           schema: (row as any).schema as string,
           table: (row as any).table as string,
         }))
-        .filter(({ schema, table }) => !allExcludes.includes(`${schema}.${table}`)),
+        .filter(({ schema, table }) => !tableNameMatch(schema, table, allExcludes)),
     );
 };
 export const tablesWithoutTrigger = (
@@ -54,8 +70,8 @@ export const tablesWithoutTrigger = (
         t.table_schema as schema,
         t.table_name as table
       FROM information_schema.tables t
-      LEFT JOIN information_schema.triggers triggers ON 
-        t.table_name = triggers.event_object_table 
+      LEFT JOIN information_schema.triggers triggers ON
+        t.table_name = triggers.event_object_table
         AND t.table_schema = triggers.event_object_schema
         AND triggers.trigger_name = substring(concat($prefix::text, lower(t.table_name), $suffix::text), 0, 64)
       WHERE
@@ -72,7 +88,7 @@ export const tablesWithoutTrigger = (
           schema: (row as any).schema as string,
           table: (row as any).table as string,
         }))
-        .filter(({ schema, table }) => !allExcludes.includes(`${schema}.${table}`)),
+        .filter(({ schema, table }) => !tableNameMatch(schema, table, allExcludes)),
     );
 };
 
@@ -90,8 +106,8 @@ export const tablesWithTrigger = (
         t.table_schema as schema,
         t.table_name as table
       FROM information_schema.tables t
-      JOIN information_schema.triggers triggers ON 
-        t.table_name = triggers.event_object_table 
+      JOIN information_schema.triggers triggers ON
+        t.table_name = triggers.event_object_table
         AND t.table_schema = triggers.event_object_schema
         AND triggers.trigger_name = substring(concat($prefix::text, lower(t.table_name), $suffix::text), 0, 64)
       WHERE
@@ -119,17 +135,17 @@ export async function runPreMigration(log: Logger, sequelize: Sequelize) {
     'set_',
     '_updated_at_sync_tick',
   )) {
+    // we need to keep the updated_at_sync_tick trigger on the changes table
+    if (schema === 'logs' && table === 'changes') {
+      continue;
+    }
+
     log.info(`Removing updated_at_sync_tick trigger from ${schema}.${table}`);
+
     await sequelize.query(
       `DROP TRIGGER set_${table}_updated_at_sync_tick ON "${schema}"."${table}"`,
     );
   }
-
-  // remove changelog trigger before migrations
-  for (const { schema, table } of await tablesWithTrigger(sequelize, 'record_', '_changelog')) {
-    log.info(`Removing changelog trigger from ${schema}.${table}`);
-    await sequelize.query(`DROP TRIGGER record_${table}_changelog ON "${schema}"."${table}"`);
-  }
 }
 
 export async function runPostMigration(log: Logger, sequelize: Sequelize) {
@@ -156,7 +172,20 @@ export async function runPostMigration(log: Logger, sequelize: Sequelize) {
       })
       .then(rows => (rows?.[0] as any)?.count > 0);
 
-  // add trigger: before insert or update, set updated_at (overriding any that is passed in)
+  // add trigger: before update, update updated_at when the data in the row changed
+  if (await functionExists('set_updated_at')) {
+    for (const { schema, table } of await tablesWithoutTrigger(sequelize, 'set_', '_updated_at')) {
+      log.info(`Adding updated_at trigger to ${schema}.${table}`);
+      await sequelize.query(`
+      CREATE TRIGGER set_${table}_updated_at
+      BEFORE INSERT OR UPDATE ON "${schema}"."${table}"
+      FOR EACH ROW
+      EXECUTE FUNCTION public.set_updated_at();
+    `);
+    }
+  }
+
+  // add trigger: before insert or update, set updated_at_sync_tick (overriding any that is passed in)
   if (await functionExists('set_updated_at_sync_tick')) {
     for (const { schema, table } of await tablesWithoutTrigger(
       sequelize,
